/* --------------------------------------------------------------------------
$Id: test_raveobject.c,v 1.4 2000/03/22 09:05:26 ahenja Exp $
Program:	testRaveObject

Description:    Simple test program for reading and writing an HDF5 file.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: test_raveobject.c,v $
Revision 1.4  2000/03/22 09:05:26  ahenja
Updated with header information.
	
-----------------------------------------------------------------------------*/
#include "vhlhdf.h"
#include "read_vhlhdf.h"
#include "write_vhlhdf.h"

typedef struct {
   int xsize;
   int ysize;
   double xscale;
   double yscale;
   double area_extent[4];
   char pcs_id[64];
   double nodata;
} RaveInfoObject;

static hid_t createRaveInfoObject()
{
   hid_t type_id=-1;
   size_t dims4[]={0,0,0,0};
   hid_t strtype=-1;

   if((type_id=createCompoundType(sizeof(RaveInfoObject)))<0)
      return -1;
   if(addAttributeToCompoundType_fmt(type_id,"xsize",HOFFSET(RaveInfoObject,xsize),"int")<0)
      goto fail;
   if(addAttributeToCompoundType_fmt(type_id,"ysize",HOFFSET(RaveInfoObject,ysize),"int")<0)
      goto fail;
   if(addAttributeToCompoundType_fmt(type_id,"xscale",HOFFSET(RaveInfoObject,xscale),"double")<0)
      goto fail;
   if(addAttributeToCompoundType_fmt(type_id,"yscale",HOFFSET(RaveInfoObject,yscale),"double")<0)
      goto fail;
   dims4[0]=4;
   if(addArrayToCompoundType_fmt(type_id,"area_extent",HOFFSET(RaveInfoObject,area_extent[0]),
				 1,dims4,"double")<0)
      goto fail;
   strtype=createStringType(64);
   if(addAttributeToCompoundType(type_id,"pcs_id",HOFFSET(RaveInfoObject,pcs_id[0]),strtype)<0)
      goto fail;
   if(addAttributeToCompoundType_fmt(type_id,"nodata",HOFFSET(RaveInfoObject,nodata),"double")<0)
      goto fail;

   if(strtype>=0)
      closeType(strtype);
   return type_id;
fail:
   if(type_id>=0)
      closeType(type_id);
   if(strtype>=0)
      closeType(strtype);
   return -1;
}

int writeRaveObject()
{
   hid_t rave_info_hid=-1;
   RaveInfoObject info;
   float* data=NULL;
   int npts,i;
   HL_NodeList* nodelist = NULL;
   HL_Node* node;
   hsize_t dims[]={0,0,0,0};

   /*Always has to do this*/
   initHlHdf();
   /*And this I just do to be able to get debugging info from hdf*/
   debugHlHdf(2);

   /* Set the region information */
   info.xsize=10;
   info.ysize=10;
   info.xscale=100.0;
   info.yscale=100.0;
   info.area_extent[0]=-500.0;
   info.area_extent[1]=-500.0;
   info.area_extent[2]=500.0;
   info.area_extent[3]=500.0;
   strcpy(info.pcs_id,"ps14e60n_sswe");
   info.nodata=255.0;

   npts=info.ysize*info.xsize;

   if((rave_info_hid=createRaveInfoObject())<0) {
      fprintf(stderr,"Failed to create rave info type\n");
      goto fail;
   }

   if(!(data=malloc(sizeof(float)*npts))) {
      fprintf(stderr,"Failed to allocate memory for data array\n");
      goto fail;
   }
   for(i=0;i<npts;i++) {
      data[i]=(float)i;
   }

   if(!(nodelist = newHL_NodeList())) {
      fprintf(stderr,"Failed to allocate nodelist\n");
      goto fail;
   }

   if(!addNode(nodelist,(node=newHL_Attribute("/PRODUCT_ID"))))
      goto fail;
   setScalarValue(node,strlen("rave_image")+1,(unsigned char*)"rave_image","string",-1);

   if(!addNode(nodelist,(node=newHL_Datatype("/RaveInfoObject"))))
      goto fail;
   commitDatatype(node,rave_info_hid);

   dims[0]=1;
   if(!addNode(nodelist,(node=newHL_Dataset("/info"))))
      goto fail;
   setArrayValue(node,sizeof(info),1,dims,(unsigned char*)&info,"compound",rave_info_hid);

   dims[0]=info.xsize;
   dims[1]=info.ysize;
   if(!addNode(nodelist,(node=newHL_Dataset("/data"))))
      goto fail;
   setArrayValue(node,sizeof(float),2,dims,(unsigned char*)data,"float",-1);

   strcpy(nodelist->filename,"rave_image_file.hdf");
   writeNodeList(nodelist,6);

fail:
   freeHL_NodeList(nodelist);
   free(data);
   return 0;
}

int readRaveObject()
{
   HL_NodeList* nodelist=readHL_NodeList("rave_image_file.hdf");
   HL_CompoundTypeDescription* descr;
   HL_Node* node;
   RaveInfoObject info;
   int i,x,y,idx;
   float* fltptr;
   HL_CompoundTypeAttribute* attr;

   /*Always has to do this*/
   initHlHdf();
   /*And this I just do to be able to get debugging info from hdf*/
   debugHlHdf(2);


   /* Ought to have a function, something like selectAll, which do
    * this. */
   selectAllNodes(nodelist);

   fetchMarkedNodes(nodelist);

   for(i=0;i<nodelist->nNodes;i++) {
      switch(nodelist->nodes[i]->type) {
      case ATTRIBUTE_ID:
	 printf("ATTRIBUTE: '%s' {\n",nodelist->nodes[i]->name);
	 if(nodelist->nodes[i]->compoundDescription) {
	    descr= findCompoundTypeDescription(nodelist,
					       nodelist->nodes[i]->compoundDescription->objno[0],
					       nodelist->nodes[i]->compoundDescription->objno[1]);
	    if(descr) {
	       printf("DATATYPE { \"%s\" }\n",descr->typename);
	    } else {
	       printf("DATATYPE { Compound type }\n");
	    }
	 }
	 printf("}\n");
	 break;
      case GROUP_ID:
	 printf("GROUP: '%s' {\n",nodelist->nodes[i]->name);
	 printf("}\n");
	 break;
      case DATASET_ID:
	 printf("DATASET: '%s' {\n",nodelist->nodes[i]->name);
	 if(nodelist->nodes[i]->compoundDescription) {
	    descr= findCompoundTypeDescription(nodelist,
					       nodelist->nodes[i]->compoundDescription->objno[0],
					       nodelist->nodes[i]->compoundDescription->objno[1]);
	    if(descr) {
	       printf("DATATYPE { \"%s\" }\n",descr->typename);
	    } else {
	       printf("DATATYPE { Compound type }\n");
	    }
	 }
	 printf("}\n");
	 break;
      case TYPE_ID:
	 printf("DATATYPE: '%s' {\n",nodelist->nodes[i]->name);
	 printf("}\n");
	 break;
      }
   }

   node = getNode(nodelist,"/info");
   printf("Size of info = %d\n",sizeof(info));
   descr=node->compoundDescription;
   if(descr) {
      printf("DESCRIPTION:\n");
      printf("SIZE=%d\n",descr->size);
      printf("NATTRS=%d\n",descr->nAttrs);
      for(i=0;i<descr->nAttrs;i++) {
	 attr = descr->attrs[i];
	 printf("ATTR = %s, FORMAT = %s, OFFSET = %d\n",attr->attrname,
		attr->format, attr->offset);
      }
   }

   memcpy(&info,node->data,sizeof(info));
   printf("INFO VALUES:\n");
   printf("XSIZE=%d\n",info.xsize);
   printf("YSIZE=%d\n",info.ysize);
   printf("XSCALE=%f\n",info.xscale);
   printf("YSCALE=%d\n",info.yscale);
   printf("AREA_EXTENT=(%f,%f,%f,%f)\n",
	  info.area_extent[0],info.area_extent[1],
	  info.area_extent[2],info.area_extent[3]);
   printf("PCS_ID=%s\n",info.pcs_id);
   printf("NODATA=%f\n",info.nodata);

   node=getNode(nodelist,"/data");
   printf("DATAFIELD:\n");
   printf("DATADIMS=%d,%d\n",node->dims[0],node->dims[1]);

   fltptr=(float*)node->data;
   for(y=0;y<node->dims[1];y++) {
      for(x=0;x<node->dims[0];x++) {
	 idx=y*node->dims[0] + x;
	 if((idx%node->dims[0])==0)
	    printf("\n");
	 printf("%.2f ",fltptr[idx]);
      }
   }
   printf("\n");

   freeHL_NodeList(nodelist);
#ifdef KALLE
   /* Code for updating one attribute,
    * some problems when trying to change the dataspace.
    */
   { /* Test update attribute */
      hid_t loc_id=openHlHdfFile("rave_image_file.hdf","rw");
      hid_t gid=H5Gopen(loc_id,".");
      hid_t attrid=H5Aopen_name(gid,"PRODUCT_ID");
      hid_t htype = H5Aget_type(attrid);
      /*size_t size = H5Tget_size(htype);*/
      /*hid_t fspace = H5Aget_space(attrid);*/
      hid_t ntype = H5Tcopy(htype);
      H5Tset_size(ntype,strlen("KALLEKULA I LINGONSKOGEN")+1);
      H5Awrite(attrid,ntype,"KALLEKULA I LINGONSKOGEN");
      H5Aclose(attrid);
      H5Gclose(gid);
      H5Tclose(htype);
      /*H5Sclose(fspace);*/
      H5Fclose(loc_id);
   }
#endif

   return 0;
}

int main(int argc, char** argv)
{
   if(argc!=2) {
      printf("Usage: %s read|write\n",argv[0]);
      exit(1);
   }
   if(strcmp(argv[1],"write")==0)
      writeRaveObject();
   else
      readRaveObject();

}
